=begin
=Assertions.rb
Standard assertions for Lapidary
=end

#<standard_header>
#
# Copyright (C) 2000-2001 Nathaniel Talbott
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#</standard_header>

module Lapidary
	module Assertions
		def assertBlock(message="")
			if (! yield)
				addFailedAssertion(message)
				throw :assertionFailed
			else
				addSuccessfulAssertion(message)
			end
		end
		def assert(boolean, message="")
			assertBlock("assert should not be called with a block!") {!block_given?}
			assertBlock(message) {boolean}
		end
		def assertEqual(expected, actual, message=nil)
			assertBlock(buildMessage(message, "Expected <FIRST> but was <SECOND>", expected, actual)) {
				expected == actual
			}
		end
		def assertRaises(exception, message="", &proc)
			actualException = nil
			assertBlock(message) {
				thrown = false
				begin
					proc.call
				rescue Exception => thrownException
					actualException = thrownException
					thrown = true
				end
				thrown
			}
			assertBlock(buildMessage(message, "Expected exception to be of type <FIRST> but was <SECOND>", exception, "#{actualException.class.name}: #{actualException.message}\n\t#{actualException.backtrace.join(\"\n\t\")}")) {
				exception == actualException.class
			}
			return actualException
		end
		def assertInstanceOf(klass, object, message="")
			assertEqual(klass, object.class, message)
		end
		def assertNil(object, message="")
			assertEqual(nil, object, message)
		end
		def assertKindOf(object, klass, message="")
			assertEqual(Class, klass.class, "The second parameter to assertKindOf should be a Class.")
			assertBlock(buildMessage(message, "Expected <FIRST> to be kind_of?<SECOND>", object, klass)) {
				object.kind_of?(klass)
			}
		end
		def assertMatch(string, re, message=nil)
			assertBlock(buildMessage(message, "Expected <FIRST> to match regexp </SECOND/>", string, re.source)) {
				string =~ re
			}
		end
		def method_missing(symbol, *arguments, &proc)
			if ( symbol.id2name =~ /assert_exception/ )
				assertRaises(*arguments, &proc)
			elsif ( symbol.id2name !~ /_[a-z]/ )
				super
			else
				send(symbol.id2name.gsub(/_([a-z])/) {$1.upcase}, *arguments, &proc)
			end
		end
		def buildMessage(original, additional, expected, actual)
			messageParts = Array.new
			messageParts << original
			arguments = [expected, actual].collect {
				| argument |
				argument.nil? ? "nil" : argument.to_s
			}
			messageParts << additional.sub(/FIRST/, arguments[0]).sub(/SECOND/, arguments[1])
			return messageParts.join(original =~ /\.$/ ? " " : ". ")
		end
			
	end
end
